#include "functions.h"
#include "icommands.h"
#include "LibLoader.h"
#include "StringBuilder.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#ifndef bool
#define bool char
#define true 1
#define false 0
#endif

#ifndef Del
#define Del(x) {free(x); x = NULL;}
#endif

#ifndef DelN
#define DelN(x) {if (x != NULL){free(x); x = NULL;}}
#endif

#ifdef __cplusplus
extern "C" {
#endif

    long current_number;
    char tmp[256], tmp2[256];

    static void __attribute__((constructor)) init() {
        current_number = 0;
    }

    /**
     * Test function, printing string
     * @param s string to print
     */
    void printString(const char * const s) {
        fprintf(stdout, "%s\n", s);
        fflush(stdout);
    }

    /**
     * Call function
     * @param function name of function to call
     * @param args string presentation of function arguments, where arguments divided by delimeter character
     * @param res string representation of result of function
     */
    void callFunction(const char * const function, const char * const args[],
            char **res) {
        current_number++;
        fprintf(stdout, "%s%c%ld%c%s%c", queryS, delimeter, current_number,
                delimeter, function, delimeter);
        int i;
        for (i = 0; args[i]; i++) {
            fprintf(stdout, "%s%c", args[i], delimeter);
        }
        fflush(stdout);
#ifdef DEBUG
        FILE *out = fopen("errors.txt", "a+");
        fprintf(out, "%s%c%ld%c%s%c\n", queryS, delimeter, current_number, delimeter, function, delimeter);
        for (i = 0; args[i]; i++) {
            fprintf(out, "%s%c", args[i], delimeter);
        }
        fclose(out);
#endif
        char *id;
        char * s = getNextString();
        if (s == NULL) {
            fprintf(stderr, "Cannot receive answer: stream closed");
            fflush(stderr);
            exit(2);
        }
        if (!treatAnswer(s, res, &id)) {
            fprintf(stderr, "Unexpected answer: %s\n", s);
            fflush(stderr);
            Del(s);
            exit(2);
        }
        long did;
        sscanf(id, "%ld", &did);
        if (did != current_number) {
            fprintf(stderr, "Unexpected identifier of answer: %s\n", id);
            fflush(stderr);
            Del(id);
            Del(*res);
            Del(s);
            exit(2);
        }
        Del(id);
        Del(s);
    }

    bool callBoolString(const char * const function, const char * const s) {
        char *sres = NULL;
        const char * const args[] = {s, NULL};
        callFunction(function, args, &sres);
        bool res = !strcmp(sres, "true");
        Del(sres);
        return res;
    }

    void callVoidString(const char * const function, const char * const s) {
        char *sres = NULL;
        const char * const args[] = {s, NULL};
        callFunction(function, args, &sres);
        Del(sres);
    }

    long callIntString(const char * const function, const char * const s) {
#ifdef DEBUG
        FILE *out = fopen("errors.txt", "a+");
        fprintf(out, "callIntString started\n");
        fclose(out);
#endif
        char *sres = NULL;
        const char * const args[] = {s, NULL};
        callFunction(function, args, &sres);
        long res;
        sscanf(sres, "%ld", &res);
        Del(sres);
        return res;
    }

    double callDoubleString(const char * const function, const char * const s) {
        char *sres = NULL;
        const char * const args[] = {s, NULL};
        callFunction(function, args, &sres);
        double res;
        sscanf(sres, "%lf", &res);
        Del(sres);
        return res;
    }

    long callIntVoid(const char * const function) {
        char *sres = NULL;
        const char * const args[] = {NULL};
        callFunction(function, args, &sres);
        long res;
        sscanf(sres, "%ld", &res);
        Del(sres);
        return res;
    }
    
    char * callStringString(const char * const function, const char * const s) {
        char *sres = NULL;
        const char * const args[] = {s, NULL};
        callFunction(function, args, &sres);
        return sres;
    }
    
    void callVoid4Int(const char * const function, long x1, long y1, long x2, long y2) {
        char *sres;
        char tm[4][256];
        tm[0][0] = '\0';
        tm[1][0] = '\0';
        tm[2][0] = '\0';
        tm[3][0] = '\0';
        sprintf(tm[0], "%ld", x1);
        sprintf(tm[1], "%ld", y1);
        sprintf(tm[2], "%ld", x2);
        sprintf(tm[3], "%ld", y2);
        const char * const args[] = {tm[0], tm[1], tm[2], tm[3], NULL};
        callFunction(function, args, &sres);
        Del(sres);
    }

    void callVoidInt(const char * const function, long d) {
        char *sres;
        tmp[0] = '\0';
        sprintf(tmp, "%ld", d);
        const char * const args[] = {tmp, NULL};
        callFunction(function, args, &sres);
        Del(sres);
    }
    
    bool callBoolVoid(const char * const function) {
        char *sres = NULL;
        const char * const args[] = {NULL};
        callFunction(function, args, &sres);
        bool res = !strcmp(sres, "true");
        Del(sres);
        return res;
    }
    
    void callVoidStringInt(const char * const function, const char * const s, long d) {
        char *tof;
        tmp[0] = '\0';
        sprintf(tmp, "%ld", d);
        const char * const args[] = {s, tmp, NULL};
        callFunction(function, args, &tof);
        Del(tof);
    }
    
    void callVoidStringDouble(const char * const function, const char * const s, double f) {
        char *tof;
        tmp[0] = '\0';
        sprintf(tmp, "%lf", f);
        const char * const args[] = {s, tmp, NULL};
        callFunction(function, args, &tof);
        Del(tof);
    }
    
    void callVoidStringBool(const char * const function, const char * const s, bool b) {
        char *tof;
        char * arg = (b) ? "true" : "false";
        const char * const args[] = {s, arg, NULL};
        callFunction(function, args, &tof);
        Del(tof);
    }
    
    void callVoidString2Int(const char * const function, const char * const s, long d1, long d2) {
        char *tof;
        tmp[0] = '\0';
        tmp2[0] = '\0';
        sprintf(tmp, "%ld", d1);
        sprintf(tmp2, "%ld", d2);
        const char * const args[] = {s, tmp, tmp2, NULL};
        callFunction(function, args, &tof);
        Del(tof);
    }

    void callVoid2ArIntInt(const char * const function, long x[], long y[], long n) {
        long argn = (1+n)*2+1;
        char **args = (char**) malloc(argn * sizeof(char*));
        int ntmp = (sizeof(long) * 8) / 3 + 1;
        char * tm = (char*) malloc(ntmp * sizeof(char*));
        tm[0] = '\0';
        sprintf(tm, "%ld", n);
        args[0] = tm;
        args[n+1] = tm;
        args[argn-1] = tm;
        long i;
        for (i = 0; i < n; i++) {
            tm = (char*) malloc(ntmp * sizeof(char*));
            tm[0] = '\0';
            sprintf(tm, "%ld", x[i]);
            args[i+1] = tm;
            tm = (char*) malloc(ntmp * sizeof(char*));
            tm[0] = '\0';
            sprintf(tm, "%ld", y[i]);
            args[i+2+n] = tm;
        }
        char *sres;
        callFunction(function, (const char * const *)args, &sres);
        for (i = 0; i < argn; i++) {
            DelN(args[i]);
        }
        DelN(args);
        Del(sres);
    }

    /**
     *  Get value of time, system variable
     *  @return time
     */
    long getTime(void) {
        return callIntVoid(__FUNCTION__);
    }
    
    /**
     * Get property of event
     * @param s name of event
     * WARNING: After get result of it functions, don't forget free memory
     * of char array
     */
    char * getProperty(const char * const s) {
        return callStringString(__FUNCTION__, s);
    }

    /**
     *  Is event sleeping?
     *  @param s name of event for checking
     *  @return true, if event sleeping and false else
     */
    bool isSleep(const char * const s) {
        return callBoolString(__FUNCTION__, s);
    }

    /**
     *  Is event running?
     *  @param s name of event for checking
     *  @return true, if event running and false else
     */
    bool isRun(const char * const s) {
        return callBoolString(__FUNCTION__, s);
    }

    /**
     *  Did event finish?
     *  @param s name of event for checking
     *  @return true, if event finished and false else
     */
    bool isFinished(const char * const s) {
        return callBoolString(__FUNCTION__, s);
    }

    /**
     *  Was event running?
     *  @param s name of event for checking
     *  @return true, if event was running and false else
     */
    bool wasEvent(const char * const s) {
        return callBoolString(__FUNCTION__, s);
    }

    /**
     *  Compel event to sleep
     *  @param s name of event
     */
    void setSleep(const char * const s) {
        callVoidString(__FUNCTION__, s);
    }

    /**
     *  Wake event
     *  @param s name of event for wake
     */
    void setWake(const char * const s) {
        callVoidString(__FUNCTION__, s);
    }

    /**
     * Kill event
     * @param s name of event to kill
     */
    void killEvent(const char * const s) {
        callVoidString(__FUNCTION__, s);
    }

    /**
     * Get delay of event
     * @param s name of event
     * @return delay of event
     */
    long getDelay(const char * const s) {
        return callIntString(__FUNCTION__, s);
    }

    /**
     * Set delay for event
     * @param s name of event
     * @param delay delay to set for event
     */
    void setDelay(const char * const s, long delay) {
        callVoidStringInt(__FUNCTION__, s, delay);
    }

    /**
     *  Get value of int variable or constant
     *  @param variable name of variable or constant
     *  @return value of variable or constant
     */
    long getInt(const char * const variable) {
#ifdef DEBUG
        FILE *out = fopen("errors.txt", "a+");
        fprintf(out, "getInt(\"%s\")\n", variable);
        fclose(out);
#endif
        return callIntString(__FUNCTION__, variable);
    }

    /**
     *  Get value of double variable or constant
     *  @param variable name of variable or constant
     *  @return value of variable or constant
     */
    double getDouble(const char * const variable) {
        return callDoubleString(__FUNCTION__, variable);
    }

    /**
     *  Get value of boolean variable or constant
     *  @param variable name of variable or constant
     *  @return value of variable or constant
     */
    bool getBoolean(const char * const variable) {
        return callBoolString(__FUNCTION__, variable);
    }
    /**
     *  Get new value for int variable
     *  @param variable name of variable
     *  @param value new value of variable
     */
    void setInt(const char * const variable, long value) {
        callVoidStringInt(__FUNCTION__, variable, value);
    }
    /**
     *  Set new value for double variable
     *  @param variable name of variable
     *  @param value new value of variable
     */
    void setDouble(const char * const variable, double value) {
        callVoidStringDouble(__FUNCTION__, variable, value);
    }
    /**
     *  Set new value for boolean variable
     *  @param variable name of variable
     *  @param value new value of variable
     */
    void setBoolean(const char * const variable, bool value) {
        callVoidStringBool(__FUNCTION__, variable, value);
    }
    
    /********************************\
     *                              *
     *  Functions for graphic mode  *
     *                              *
    \********************************/

    /**
     *  Get height of window
     *  @return height
     */
    long getHeight(void) {
        return callIntVoid(__FUNCTION__);
    }
    /**
     *  Get width of window
     *  @return width
     */
    long getWidth() {
        return callIntVoid(__FUNCTION__);
    }

    /**
     * Has image in window?
     * @return true, if has, false otherwise
     */
    bool hasImage(void) {
        return callBoolVoid(__FUNCTION__);
    }
    /**
     *  Draw the line
     *  @param x1 the first point's x coordinate
     *  @param y1 the first point's y coordinate
     *  @param x2 the second point's x coordinate
     *  @param y2 the second point's y coordinate
     */
    void drawLine(long x1, long y1, long x2, long y2) {
        callVoid4Int(__FUNCTION__, x1, y1, x2, y2);
    }
    /**
     *  Draw the string on window
     *  @param s string for drawing
     *  @param x the x coordinate
     *  @param y the y coordinate
     */
    void drawString(const char * const s, long x, long y) {
        callVoidString2Int(__FUNCTION__, s, x, y);
    }
    /**
     *  Draw oval
     *  @param x the center's x coordinate
     *  @param y the center's y coordinate
     *  @param width width of oval
     *  @param height height of oval
     */
    void drawOval(long x, long y, long width, long height) {
        callVoid4Int(__FUNCTION__, x, y, width, height);
    }
    /**
     *  Draw fill oval
     *  @param x the center's x coordinate
     *  @param y the center's y coordinate
     *  @param width width of oval
     *  @param height height of oval
     */
    void fillOval(long x, long y, long width, long height) {
        callVoid4Int(__FUNCTION__, x, y, width, height);
    }
    /**
     *  Fills the specific rectangle, where
     *  (x,y) - the left top point of rectangle
     *  @param x the left top point's x coordinate
     *  @param y the left top point's y coordinate
     *  @param width the width of rectangle
     *  @param height the height of rectangle
     */
    void fillRect(long x, long y, long width, long height) {
        callVoid4Int(__FUNCTION__, x, y, width, height);
    }
    /**
     *  Draw the fill polynom
     *  @param x[] the array of x coordinates of points
     *  @param y[] the array of y coordinates of points
     *  @param n number of points in polynom
     */
    void fillPolygon(long x[], long y[], long n) {
        callVoid2ArIntInt(__FUNCTION__, x, y, n);
    }
    /**
     *  Set value of current color
     *  @param c color number in RGB code
     */
    void setColor(long c) {
        callVoidInt(__FUNCTION__, c);
    }
    /**
     *  Return the current color in RGB code
     *  @return current color
     */
    long getColor() {
        return callIntVoid(__FUNCTION__);
    }

#ifdef __cplusplus
}
#endif
