#ifndef STRINGBUILDER_H
#define STRINGBUILDER_H
#include <stdio.h>

/**
 * Struct of StringBuilder
 */
struct StringBuilder {
	char **array;//Array of strings, max len of strings = StringBuilder_MAX
	int last_string;//Index of last string, starts from 0
	int character_pos;//Index of next character in last_string to add
#ifndef StringBuilder_MAX
#define StringBuilder_MAX 256
#endif
};

#ifndef P
#define P(x) struct x *
#endif

/**
 * Constructor of StringBuilder
 * @param p pointer to StringBuilder
 */
extern void sbInit(P(StringBuilder) p);
/**
 * Check for empty StringBuilder
 * @return non-zero value, if StringBuilder is empty, 0 else
 */
extern int sbIsEmpty(P(StringBuilder) p);
/**
 * Create new instance of StringBuilder
 * @return pointer to new instance of StringBuilder
 */
extern P(StringBuilder) sbNew();
/**
 * Destructor of StringBuilder
 * @param p pointer to pointer to StringBuilder
 */
extern void sbFinit(P(StringBuilder) *p);
/**
 * Get string from StringBuilder
 * @param p pointer to StringBuilder
 */
extern char * sbToString(P(StringBuilder) p);
/**
 * Add character to StringBuilder
 * @param ch character to add
 * @param p pointer to StringBuilder
 */
extern void sbAddChar(P(StringBuilder) p, const char ch);
/**
 * Add string to StringBuilder
 * @param s string to add
 * @param p pointer to StringBuilder
 */
extern void sbAddString(P(StringBuilder) p, const char * const s);

/**
 * Print content of StringBuilder for debug purpose to out file pointer
 * @param out pointer to file for output
 * @param p pointer to StringBuilder
 */
extern void sbPrintOut(P(StringBuilder) p, FILE * out);

/**
 * Print content of StringBuilder for debug purpose to stdout
 * @param p pointer to StringBuilder
 */
extern void sbPrint(P(StringBuilder) p);
#endif
