#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#ifndef bool
#define bool char
#define true 1
#define false 0
#endif

#ifdef __cplusplus
extern "C" {
#endif
    /**
     * Test function, printing string
     * @param s string to print
     */
    extern void printString(const char * const s);
    /**
     * Get property of event
     * @param s name of event
     * WARNING: After get result of it functions, don't forget free memory
     * of char array
     */
    extern char * getProperty(const char * const s);
    /**
     *  Get value of time, system variable
     *  @return time
     */
    extern long getTime(void);
    /**
     *  Is event sleeping?
     *  @param s name of event for checking
     *  @return true, if event sleeping and false else
     */
    extern bool isSleep(const char * const s);
    /**
     *  Is event running?
     *  @param s name of event for checking
     *  @return true, if event running and false else
     */
    extern bool isRun(const char * const s);
    /**
     *  Did event finish?
     *  @param s name of event for checking
     *  @return true, if event finished and false else
     */
    extern bool isFinished(const char * const s);
    /**
     *  Was event running?
     *  @param s name of event for checking
     *  @return true, if event was running and false else
     */
    extern bool wasEvent(const char * const s);
    /**
     *  Compel event to sleep
     *  @param s name of event
     */
    extern void setSleep(const char * const s);
    /**
     *  Wake event
     *  @param s name of event for wake
     */
    extern void setWake(const char * const s);
    /**
     * Kill event
     * @param s name of event to kill
     */
    extern void killEvent(const char * const s);
    /**
     * Get delay of event
     * @param s name of event
     * @return delay of event
     */
    extern long getDelay(const char * const s);
    /**
     * Set delay for event
     * @param s name of event
     * @param delay delay to set for event
     */
    extern void setDelay(const char * const s, long delay);
    /**
     *  Get value of int variable or constant
     *  @param variable name of variable or constant
     *  @return value of variable or constant
     */
    extern long getInt(const char * const variable);
    /**
     *  Get value of double variable or constant
     *  @param variable name of variable or constant
     *  @return value of variable or constant
     */
    extern double getDouble(const char * const variable);
    /**
     *  Get value of boolean variable or constant
     *  @param variable name of variable or constant
     *  @return value of variable or constant
     */
    extern bool getBoolean(const char * const variable);
    /**
     *  Get new value for int variable
     *  @param variable name of variable
     *  @param value new value of variable
     */
    extern void setInt(const char * const variable, long value);
    /**
     *  Set new value for double variable
     *  @param variable name of variable
     *  @param value new value of variable
     */
    extern void setDouble(const char * const variable, double value);
    /**
     *  Set new value for boolean variable
     *  @param variable name of variable
     *  @param value new value of variable
     */
    extern void setBoolean(const char * const variable, bool value);
    /********************************\
     *                              *
     *  Functions for graphic mode  *
     *                              *
    \********************************/

    /**
     *  Get height of window
     *  @return height
     */
    extern long getHeight();
    /**
     *  Get width of window
     *  @return width
     */
    extern long getWidth();
    /**
     *  Has any image for window graphic?
     *  @return true, if image existing and false otherwise
     */
    extern bool hasImage();
    /**
     *  Draw the line
     *  @param x1 the first point's x coordinate
     *  @param y1 the first point's y coordinate
     *  @param x2 the second point's x coordinate
     *  @param y2 the second point's y coordinate
     */
    extern void drawLine(long x1, long y1, long x2, long y2);
    /**
     *  Draw the string on window
     *  @param s string for drawing
     *  @param x the x coordinate
     *  @param y the y coordinate
     */
    extern void drawString(const char * const s, long x, long y);
    /**
     *  Draw oval
     *  @param x the center's x coordinate
     *  @param y the center's y coordinate
     *  @param width width of oval
     *  @param height height of oval
     */
    extern void drawOval(long x, long y, long width, long height);
    /**
     *  Draw fill oval
     *  @param x the center's x coordinate
     *  @param y the center's y coordinate
     *  @param width width of oval
     *  @param height height of oval
     */
    extern void fillOval(long x, long y, long width, long height);
    /**
     *  Fills the specific rectangle, where
     *  (x,y) - the left top point of rectangle
     *  @param x the left top point's x coordinate
     *  @param y the left top point's y coordinate
     *  @param width the width of rectangle
     *  @param height the height of rectangle
     */
    extern void fillRect(long x, long y, long width, long height);
    /**
     *  Draw the fill polynom
     *  @param x[] the array of x coordinates of points
     *  @param y[] the array of y coordinates of points
     *  @param n number of points in polynom
     */
    extern void fillPolygon(long x[], long y[], long n);
    /**
     *  Set value of current color
     *  @param c color number in RGB code
     */
    extern void setColor(long c);
    /**
     *  Return the current color in RGB code
     *  @return current color
     */
    extern long getColor();

#ifdef __cplusplus
}
#endif
#endif
